"""Test utility modules."""
import json
import os


def load_json(filepath: str):
    """Load `*.json` test file."""
    if not filepath:
        return None

    abs_path = _resolve_relative_path(filepath)
    with open(abs_path) as f:
        raw_json = f.read()

        return json.loads(raw_json)


def _resolve_relative_path(filepath: str):
    """Resolve relative import path."""
    if not filepath:
        return None

    inf_path = os.path.join(os.path.dirname(__file__), filepath)

    return inf_path



"""
Facade for `upcoming_events` Delegation

Bet365 API Responses contain 2 common components
Bet365Response object is tasked to parse:
    "success": int
    "results": list

This module provides delegation for `UpcomingEventsResponse`

The objects are accessible via dot notation or via `.get(..)`
"""

from typing import List, Union

from pybet365.response.base import (
    Bet365Response,
    PagerBase,
    ResultBase
)


class UpcomingEvent(ResultBase):
    """
    Component for dot notation access of `results` from UpcomingEvent Endpoint.

    (e.g.)
    >>> data = {
    ...   "id":"86576599",
    ...   "sport_id":"1",
    ...   "away":{
    ...       "id":"10361085",
    ...       "name":"Chivas Guadalajara Women"
    ...    },
    ...    "ss": None,
    ...    "our_event_id":"2130836",
    ...    "updated_at":"1581990232"
    ... }

    >>> UpcomingEvent(data).id
    >>> "86576599"

    >>> UpcomingEvent(data).away.id
    >>> "10361085"
    """

    def __init__(self, data: dict):
        """Constructor for UpcomingEvent."""
        super(UpcomingEvent, self).__init__(data)

    @property
    def our_event_id(self) -> str:
        """Access for `our_event_id`."""
        return self.get("our_event_id")

    @property
    def updated_at(self) -> str:
        """Access for `updated_at`."""
        return self.get("updated_at")


class UpcomingEventsResponse(Bet365Response):
    """
    Response Object Facade for UpcomingEvents Endpoint.

    The object wraps the response and exposes dot notation access

    The top level accesses for `upcoming` endpoint are:
        "success": int
        "pager": dict
        "results": list

    The `results` object is parsed into `UpcomingEvent` facades
    Say you have a parsed response from UpcomingEvents Endpoint
    >>> response_object.results
    >>> [
    ...   {
    ...     "id": "12345",
    ...     "our_event_id":"2294461",
    ...     "updated_at": "1586461906"
    ...   },
    ... ]

    >>> response_object.results[0].our_event_id
    >>> "2294461"

    >>> response_object.results[0].updated_at
    >>> "1586461906"
    """

    def __init__(self, data):
        """Constructor for UpcomingEventsResponse."""
        super(UpcomingEventsResponse, self).__init__(data)
        self.pager = PagerBase(data.get("pager"))

    @property
    def results(self) -> Union[List[UpcomingEvent], None]:
        """Access for `results`."""
        if not self._results:
            return None

        parsed_results = []
        for record in self._results:
            parsed_results.append(UpcomingEvent(record))

        return parsed_results



"""Unit tests for `betfund_bet365.client` modules."""
from unittest import TestCase

from pybet365.response.base import (
    Bet365Response,
    MetaBase,
    PagerBase,
    ResultBase,
    StatsBase,
)

from tests.utils import load_json


class TestBet365Response(TestCase):
    """Unit tests for Bet365Response Facade."""

    def setUp(self) -> None:
        """Instantiate Bet365Response."""

        self.test_client = Bet365Response(
            load_json("testData/upcoming_events_table_tennis.json")
        )

    def test_success(self):
        """Unit test for `Bet365Response.success`."""
        result = self.test_client.success

        assert result == 1

    def test_results(self):
        """Unit test for `Bet365Response._results`."""
        result = self.test_client._results

        assert isinstance(result, list)
        assert len(result) == 1


class TestMetaBase(TestCase):
    """Unit tests for MetaBase Facade."""

    def setUp(self) -> None:
        """Instantiate MetaBase."""
        mock_meta = {
            "id": "10423098",
            "name": "Evgenii Kryuchkov",
            "image_id": "1238012",
            "cc": "Russia",
        }

        self.test_client = MetaBase(mock_meta)

    def test_id(self):
        """Unit test for `MetaBase.id`."""
        result = self.test_client.id

        assert result == "10423098"

    def test_name(self):
        """Unit test for `MetaBase.name`."""
        result = self.test_client.name

        assert result == "Evgenii Kryuchkov"

    def test_image_id(self):
        """Unit test for `MetaBase.name`."""
        result = self.test_client.image_id

        assert result == "1238012"

    def test_cc(self):
        """Unit test for `MetaBase.name`."""
        result = self.test_client.cc

        assert result == "Russia"


class TestPagerBese(TestCase):
    """Unit tests for PagerBase Facade."""

    def setUp(self) -> None:
        """Instantiate MetaBase."""
        mock_pager = {
            "page": 1,
            "per_page": 50,
            "total": 730
        }

        self.test_client = PagerBase(mock_pager)

    def test_page(self):
        """Unit test for `PagerBase.page`."""
        result = self.test_client.page

        assert result == 1

    def test_per_page(self):
        """Unit test for `PagerBase.page`."""
        result = self.test_client.per_page

        assert result == 50

    def test_total(self):
        """Unit test for `PagerBase.total`."""
        result = self.test_client.total

        assert result == 730


class TestResultBase(TestCase):
    """Unit tests for ResultBase Facade."""

    def setUp(self) -> None:
        """Instantiate ResultBase."""
        mock_result = {
            "id": "86576599",
            "sport_id": "1",
            "time": "15093049123",
            "time_status": "12312",
            "league": {
                "id": "10361086",
                "name": "Chivas Guadalajara"
            },
            "away": {
                "id": "10361085",
                "name": "Chivas Guadalajara Women"
            },
            "home": {
                "id": "10361086",
                "name": "Chivas Guadalajara Men"
            },
            "ss": None,
            "our_event_id": "2130836",
            "updated_at": "1581990232"
        }

        self.test_client = ResultBase(mock_result)

    def test_id(self):
        """Unit test for `ResultBase.id`"""
        result = self.test_client.id

        assert result == "86576599"

    def test_sport_id(self):
        """Unit test for `ResultBase.sport_id`"""
        result = self.test_client.sport_id

        assert result == "1"

    def test_time(self):
        """Unit test for `ResultBase.time`"""
        result = self.test_client.time

        assert result == "15093049123"

    def test_time_status(self):
        """Unit test for `ResultBase.time`"""
        result = self.test_client.time_status

        assert result == "12312"

    def test_time_league(self):
        """Unit test for `ResultBase.league`"""
        result = self.test_client.league

        assert isinstance(result, dict)

    def test_time_away(self):
        """Unit test for `ResultBase.time`"""
        result = self.test_client.away

        assert isinstance(result, dict)

    def test_time_home(self):
        """Unit test for `ResultBase.time`"""
        result = self.test_client.home

        assert isinstance(result, dict)

    def test_time_ss(self):
        """Unit test for `ResultBase.ss`"""
        result = self.test_client.ss

        assert result is None


class TestStatsBase(TestCase):
    """Unit tests for StatsBase Facade."""

    def setUp(self) -> None:
        """Instantiate StatsBase."""
        mock_stats = {
            "event_id": "2130389",
            "update_at": "1581990853",
            "update_dt": "2020-02-18 01:54:13",
        }

        self.test_client = StatsBase(mock_stats)

    def test_event_id(self):
        """Unit test for `StatsBase.event_id`."""
        result = self.test_client.event_id

        assert result == "2130389"

    def test_update_at(self):
        """Unit test for `StatsBase.update_at`."""
        result = self.test_client.update_at

        assert result == "1581990853"

    def test_update_dt(self):
        """Unit test for `StatsBase.update_dt`."""
        result = self.test_client.update_dt

        assert result == "2020-02-18 01:54:13"



"""Configuration File for Bet365 Client."""
from enum import DynamicClassAttribute, Enum

from typing import Union


class ExtendedEnum(Enum):
    """Allows access to overloaded functions for standard Enum"""

    @classmethod
    def list(cls):
        """Interface to provide all allowed values for a RundownSportId. """
        return list(map(lambda c: (c.sport_id, c.pretty), cls))


class Bet365SportId(ExtendedEnum):
    """
    Bet365 API SportId Enumeration.

    Registered members have accesses to:
    _name_ - name
    _value_ - value[0]
    _pretty_ - value[1]

    (e.g.)
    >>> soccer = Bet365SportId.AMERICAN_FOOTBALL

    >>> soccer.name
    >>> "AMERICAN_FOOTBALL"

    >>> soccer.value
    >>> 12

    >>> soccer.pretty
    >>> "american-football"
    """

    @DynamicClassAttribute
    def pretty(self) -> Union[str, None]:
        """Access for `pretty` in base `Enum`."""
        return self.value[1] if self else None

    @DynamicClassAttribute
    def sport_id(self) -> Union[str, None]:
        """Access for `sport_id` in base `Enum`."""
        return self.value[0] if self else None

    SOCCER = ("1", "soccer")
    CRICKET = ("3", "cricket")
    RUGBY_UNION = ("8", "rugby-union")
    BOXING_UFC = ("9", "boxing-ufc")
    AMERICAN_FOOTBALL = ("12", "american-football")
    TENNIS = ("13", "tennis")
    SNOOKER = ("14", "snooker")
    DARTS = ("15", "darts")
    BASEBALL = ("16", "baseball")
    ICE_HOCKEY = ("17", "ice-hockey")
    BASKETBALL = ("18", "basketball")
    RUGBY_LEAGUE = ("19", "rugby")
    AUSTRALIAN_RULES = ("36", "austalian-rules")
    BOWLS = ("66", "bowls")
    GAELIC_SPORTS = ("75", "gaelic-sports")
    HANDBALL = ("78", "handball")
    FUTSAL = ("83", "futsal")
    FLOORBALL = ("90", "floorball")
    VOLLEYBALL = ("91", "volleyball")
    TABLE_TENNIS = ("92", "table-tennis")
    BADMINTON = ("94", "badminton")
    BEACH_VOLLEYBALL = ("95", "beach-volleyball")
    SQUASH = ("107", "squash")
    WATER_POLO = ("110", "water-polo")
    E_SPORTS = ("151", "e-sports")


class Bet365Mnemonic(Enum):
    """
    Bet365 API Mnemonic Field Enumeration.
    see: (https://1394697259.gitbook.io/bet365-api/bet36)

    Internal mapping to cast fields where necessary
    """

    PLACE_365 = "3P"
    WIN_365 = "3W"
    MARKET_GROUP_PAIR_ID = "4Q"
    FINANCIALS_PRICE_1 = "AB"
    STATS_COLUMN = "AC"
    ADDITIONAL_DATA_TEAM_TOUCHDOWN_QUOTE = "AD"
    STATS_CELL = "AE"
    ARCHIVE_FIXTURE_INFO = "AF"
    ASIAN_HOVER_FINANCIALS_MARKET_ODDS_1 = "AH"
    ANIMATION_ID = "AI"
    FINANCIALS_MARKET_ODDS_2 = "AJ"
    ANIMATION_ICON = "AM"
    ANIMATION_TOPIC = "AO"
    STATS_PANE = "AP"
    FINANCIALS_CLOSE_TIME = "AQ"
    ADDITIONAL_STATS_ANIMATION_SOUND_TEAM_FIELDGOAL_QUOTE = "AS"
    ANIMATION_TEXT_STATS_TAB = "AT"
    AUDIO_AVAILABLE = "AU"
    ARCHIVE_VIDEO_AVAILABLE = "AV"
    BUTTON_BAR = "BB"
    BOOK_CLOSES_CLOSE_BETS_COUNT = "BC"
    PULL_BET_DATA = "BD"
    BET = "BE"
    BLURB_HEADER = "BH"
    BUTTON_BAR_INDEX_BUTTON_SPLIT_INDEX = "BI"
    BASE_LINE = "BL"
    BASE_ODDS_OPEN_BETS_COUNT = "BO"
    BANNER_STYLE = "BS"
    INFO_POD_DETAIL_2 = "BT"
    C1_ID_MINI_DIARY_C1 = "C1"
    C2_ID_MINI_DIARY_C2 = "C2"
    MINI_DIARY_C3 = "C3"
    CLOSE_BETS = "CB"
    BET_TYPE_PULL_COMPETITION_CODE = "CC"
    COMPETITION_DROPDOWN_FINANCIALS_TRADE = "CD"
    CONFIG = "CF"
    GLOBAL_CONFIG = "CG"
    CLASS_ID_MINI_DIARY_CUP_ICON = "CI"
    COMPETITION_KEY = "CK"
    CLASSIFICATION = "CL"
    BET_CALL_FEATURE_DISABLED_COMMENT = "CM"
    CHANNEL_COLUMN_NUMBER = "CN"
    COLUMN = "CO"
    CLOSE_BETS_PRESENTATION_PULL_DISABLED_CURRENT_PROGRESS_CURRENT_PERIOD = "CP"  # noqa: E501
    CLASS_ORDER_CLOSE_BET_RETURNS = "CR"
    CLASSIFICATIONS = "CS"
    COMPETITION_NAME = "CT"
    CURRENT_INFO = "CU"
    DATA_1 = "D1"
    DATA_2 = "D2"
    DATA_3 = "D3"
    DATA_4 = "D4"
    DATA_5 = "D5"
    DIARY_DAY = "DA"
    DISPLAY_CLOCK = "DC"
    DISPLAY_DATE = "DD"
    DESCRIPTION = "DE"
    IN_PLAY_LAUNCHER_DISPLAY_MODE = "DM"
    DIARY_NAME_DRAW_NUMBER = "DN"
    DEFAULT_OPEN = "DO"
    DECIMAL_PLACES = "DP"
    DIARY_REFRESH = "DR"
    DISPLAY_SCORE = "DS"
    DISABLE_COLUMN_DISTRIBUTION = "DX"
    DIARY = "DY"
    EVENT_TIME = "EA"
    ERROR_CODE_EXCLUDED_COUNTRY_CODES = "EC"
    EXTRA_DATA_2_TEAM_ODDS_A = "ED"
    ETOTE_LINK_DATA = "EE"
    EVENT_ID = "EI"
    EXTRA_STATS_AVAILABLE = "EL"
    EMPTY = "EM"
    EXTRA_PARTICIPANTS = "EP"
    ERROR_LOGGING = "ER"
    EMBEDDED_STREAMING_EXTRA_SCORES = "ES"
    END_TIME_EVENT_TYPE = "ET"
    EVENT = "EV"
    EACH_WAY = "EW"
    EXTRA_DATA_1_TEAM_ODDS_H = "EX"
    FORCE_DISPLAY = "FD"
    FILTERING = "FF"
    FIXTURE_PARENT_ID = "FI"
    FINANCIALS_FEED_1 = "FK"
    FINANCIALS_PERIOD_1_APN_FLUC = "FL"
    FINANCIALS_MARKET_1A = "FM"
    FINANCIALS_MARKET_1B = "FN"
    FINANCIALS_FEED_2_FORM_PULL = "FO"
    FINANCIALS_PERIOD_2_FIXED_PLACE = "FP"
    FINANCIALS_MARKET_2A = "FQ"
    FINANCIALS_MARKET_2B = "FR"
    FIXTURE_STARTED = "FS"
    FIXED_WIN = "FW"
    LOTTO_GAME_CODE = "GC"
    LOTTO_GAME_MARKET = "GM"
    GROUP = "GR"
    HANDICAP = "HA"
    HANDICAP_FORMATTED = "HD"
    HEADER_IMAGE_BET_HISTORY = "HI"
    MARKET_BAR = "HM"
    DEFAULT_OPEN_HOMEPAGE = "HO"
    SHOW_ON_HOMEPAGE = "HP"
    HASH = "HS"
    POD_HEADER_TEXT = "HT"
    INFO_BANNER_SUBHEAD2 = "HU"
    POD_BODY_TEXT_2 = "HV"
    HORSE_WEIGHT = "HW"
    HORSE_AGE = "HY"
    ID2 = "I2"
    AUDIO_ICON_DIARY_AUDIO_AVAILABLE = "IA"
    IBOX = "IB"
    ICON = "IC"
    ID = "ID"
    IN_PLAY = "IF"
    IMAGE_ID = "IG"
    IMAGE_INCLUDE_OVERVIEW_MARKET = "IM"
    INFO_INFO_POD_IMAGE_URL = "IN"
    ITEM_ORDER = "IO"
    IN_PLAY_AVAILABLE_FLAG_PARENT_ID = "IP"
    INFO_POD_IMAGE1 = "IQ"
    INRUNNING_INFO = "IR"
    INFO_POD_IMAGE_PATH1 = "IS"
    TOPIC_ID = "IT"
    INFO_POD_IMAGE2 = "IU"
    JOCKEY_PULL = "JN"
    JOCKEY = "JY"
    KIT_COLORS = "KC"
    KIT_ID = "KI"
    BREADCRUMB_LEVEL_1 = "L1"
    LABEL_INFO_POD_LINK_1_ID = "LA"
    INFO_POD_LINK_1_DISPLAY_TEXT = "LB"
    EVENT_COUNT_INFO_POD_LINK_1_C1_ID = "LC"
    INFO_POD_LINK_1_C1_ID_TABLE = "LD"
    INFO_POD_LINK_1_C2_ID = "LE"
    INFO_POD_LINK_1_C2_ID_TABLE = "LF"
    INFO_POD_LINK_2_ID_SOCCER_LEAGUE = "LG"
    INFO_POD_LINK_2_DISPLAY_TEXT = "LH"
    INFO_POD_LINK_2_C1_ID = "LI"
    INFO_POD_LINK_2_C1_ID_TABLE = "LJ"
    INFO_POD_LINK_2_C2_ID = "LK"
    INFO_POD_LINK_2_C2_ID_TABLE = "LL"
    POD_ENCODED_URL_1_LIVE_MARKETS = "LM"
    POD_ENCODED_URL_2 = "LN"
    DEFAULT_OPEN_LEFT = "LO"
    LIVE_IN_PLAY_INFO_POD_LINK_1_C3_ID = "LP"
    INFO_POD_LINK_1_C3_ID_TABLE = "LQ"
    INFO_POD_LINK_1_C3_SECTION_ID_LAST_RACES = "LR"
    PREVIOUS_SET_SCORE_SELECTED = "LS"
    MARKET = "MA"
    BET_CALL_V2_DISABLED_MAX_BET = "MB"
    CUSTOMER_TO_CUSTOMER_CALLING_FEATURE_DISABLED_COMMENT_V4_MARKET_COUNT = "MC"  # noqa: E501
    MATCHLIVE_PERIOD = "MD"
    MULTI_EVENT = "ME"
    MATCH_FLAG = "MF"
    MARKET_GROUP = "MG"
    MATCH_LENGTH = "ML"
    MERGE_MARKET = "MM"
    SECONDARY_UK_EVENT = "MO"
    MATCH_POSTPONED = "MP"
    CUSTOMER_TO_REPRESENTATIVE_CALLING_FEATURE_DISABLED_MORE_MARKETS = "MR"  # noqa: E501
    MEDIA_ID = "MS"
    BET_CALL_V2_TWILIO_DISABLED_MARKET_TYPE = "MT"
    MULTILINE = "MU"
    LOTTO_MAX_WINNINGS = "MW"
    MARKET_STYLE = "MY"
    NAME2 = "N2"
    NAME = "NA"
    CLOTH_NUMBER = "NC"
    NGENERA = "NG"
    NEXT_HEADER = "NH"
    NON_MATCH_BASED = "NM"
    NON_RUNNER = "NR"
    NEUTRAL_VENUE_TEXT = "NT"
    NEUTRAL_VENUE = "NV"
    BANKER_OPTION_OPEN_BETS_ENABLED = "OB"
    ODDS = "OD"
    ODDS_HISTORY = "OH"
    ODDS_OVERRIDE = "OO"
    OPEN_BETS_PRESENTATION_PULL_DISABLED_OPEN_BETS = "OP"
    ORDER = "OR"
    OTHERS_AVAILABLE = "OT"
    PARTICIPANT = "PA"
    PUSH_BALANCE_ENABLED = "PB"
    PAGE_DATA_1_PARTICIPANT_COUNT_PARTIAL_CASHOUT_AVAILABLE = "PC"
    PAGE_DATA_POD_INFO_POD_TYPE_PULL_DELAY = "PD"
    PARTICIPANTS_EXCEEDED_PERIOD = "PE"
    PUSH_FLAG = "PF"
    PENALTY_GOALS_MATCHLIVE_ADDITIONAL_INFO_PAGE_TYPE = "PG"
    PHONE_ONLY = "PH"
    PLAYING_INDICATOR_AUS_TOTE_COMBINATION = "PI"
    CLOTH_NUMBER_PULL = "PN"
    POD_STACK_ORDER_POINTS = "PO"
    POD_OPEN = "PP"
    PREFERENCE_ID_MARKET_GROUP_USER_PREFERENCE = "PR"
    POD_STACK_PARTICIPANT_STATUS = "PS"
    PRODUCT_TYPE_POD_TYPE = "PT"
    PREMIUM_VERSION = "PV"
    NO_OFFER = "PX"
    PARTICIPANT_STYLE = "PY"
    RANGE = "RA"
    RESULT_CODE = "RC"
    RACE_DETAILS = "RD"
    BET_RETURNS = "RE"
    REGION = "RG"
    R4_COMMENT = "RI"
    DEFAULT_OPEN_RIGHT_RACE_OFF = "RO"
    RUNNER_STATUS_REGULAR_SINGLE = "RS"
    RESULTS_TEXT = "RT"
    MATCHLIVE_STATS_1 = "S1"
    MATCHLIVE_STATS_2 = "S2"
    MATCHLIVE_STATS_3 = "S3"
    MATCHLIVE_STATS_4 = "S4"
    MATCHLIVE_STATS_5 = "S5"
    MATCHLIVE_STATS_6 = "S6"
    MATCHLIVE_STATS_7 = "S7"
    MATCHLIVE_STATS_8 = "S8"
    CHANGE_STAMP_SUSPEND_ARRAY = "SA"
    SCOREBOARD_TYPE = "SB"
    SCORE_SCORES_COLUMN = "SC"
    AUDIO_ID = "SD"
    SECONDARY_EVENT = "SE"
    SPOTLIGHT_FORM = "SF"
    STAT_GROUP = "SG"
    IMAGE_ID_PULL_SECTION_ID = "SI"
    SCORES_CELL = "SL"
    START_TIME = "SM"
    DRAW_NUMBER_PULL = "SN"
    STAT_PERIOD = "SP"
    SHORT_SCORE_SUSPENDED_SELECTION = "SS"
    INFO_POD_DETAIL_1_STAT_POD_BODY_TEXT_1_STAKE = "ST"
    SUCCESS_SUSPENDED = "SU"
    MATCHLIVE_AVAILABLE = "SV"
    STYLE = "SY"
    STAT_LOCATION = "SZ"
    C1_TABLE_MINI_DIARY_T1_TEXT_1 = "T1"
    C2_TABLE_MINI_DIARY_T2_TEXT_2 = "T2"
    MINI_DIARY_T3_TEXT_3 = "T3"
    TEXT_4 = "T4"
    TEXT_5 = "T5"
    TIME_ADDED = "TA"
    BREADCRUMB_TRAIL = "TB"
    BET_TOTE_TYPE_TEAM_COLOR = "TC"
    COUNTDOWN_TAX_DETAILS = "TD"
    TEAM = "TE"
    TEAM_GROUP = "TG"
    TMR_SERVER = "TI"
    LEAGUE_TOPIC_TOPIC_LIST = "TL"
    STAT_TIME_TMR_MINS = "TM"
    TRAINER_NAME = "TN"
    EMPTY_TOPIC_ID_PHONE_ONLY_LIST = "TO"
    TIME_STAMP = "TP"
    TAX_RATE_TOPIC_REFERENCE = "TR"
    TMR_SECS_TOTE_NAMES = "TS"
    TMR_TICKING = "TT"
    TMR_UPDATED = "TU"
    TAX_METHOD_TOPIC_LIST_EXCLUSIONS = "TX"
    CURRENT_INFO_V4 = "UC"
    UPDATE_FREQUENCY = "UF"
    VALUE = "VA"
    MATCHLIVE_ANIMATION = "VC"
    VIRTUAL_DATA = "VD"
    VIDEO_AVAILABLE = "VI"
    VISIBLE = "VL"
    VIRTUAL_RACE = "VR"
    VIDEO_STREAM = "VS"
    WIZE_GUY = "WG"
    WINNING_MARGIN = "WM"
    CHECK_BOX = "XB"
    EXCLUDE_COLUMN_NUMBERS = "XC"
    EXTRA_INFO_NODE_TEAM_MATCHTOTAL_QUOTE = "XI"
    CONTROLLER = "XL"
    SHORT_POINTS = "XP"
    EXTRA_TIME_LENGTH = "XT"
    MATCHLIVE_COORDINATES = "XY"
    TIMEZONE_ADJUSTMENT = "ZA"
    PADDOCK_VIDEO_AVAILABLE = "_V"


RESPONSE_OBJECT_FACTORY = {
    "result": "ResultResponse",
    "inplay_filter": "InPlayFilterResponse",
    "event": "InPlayOddsResponse",
    "prematch": "PreMatchOddsResponse",
    "inplay": "InPlayEventsResponse",
    "upcoming": "UpcomingEventsResponse",
}



"""
Facade for `pre_match_odds` Delegation

Bet365 API Responses contain 2 common components
Bet365Response object is tasked to parse:
    "success": int
    "results": list

This module provides delegation for `PreMatchOddsResponse`

The objects are accessible via dot notation or via `.get(..)`
"""

from typing import List, Union

from pybet365.response.base import (
    Bet365Response,
    FiResultBase
)


class PreMatchOddsResponse(Bet365Response):
    """
    Response Object Facade for Result Endpoint.

    The object wraps the response and exposes dot notation access

    The top level accesses for `upcoming` endpoint are:
        "success": int
        "results": list

    The `results` object is parsed into `FiResultBase` facades
    Say you have a parsed response from UpcomingEvents Endpoint
    >>> response_object.results
    >>> [
    ...   {
    ...     "id": "67890",
    ...     "inplay_created_at": "1586461906"
    ...   },
    ... ]

    >>> response_object.results[0].id
    >>> "67890"

    >>> response_object.results[0].inplay_created_at
    >>> "1576465906"
    """

    def __init__(self, data):
        """Constructor for PreMatchOddsResponse."""
        super(PreMatchOddsResponse, self).__init__(data)

    @property
    def results(self) -> Union[List[FiResultBase], None]:
        """Access for `events`."""
        if not self._results:
            return None

        parsed_results = []
        for record in self._results:
            parsed_results.append(FiResultBase(record))

        return parsed_results



"""
Facade for `in_play_events` Delegation

Bet365 API Responses contain 2 common components
Bet365Response object is tasked to parse:
    "success": int
    "results": list

This module provides delegation for `InPlayEventsResponse`

The objects are accessible via dot notation or via `.get(..)`
"""

from typing import List, Union

from pybet365.response.base import (
    Bet365Response,
    StatsBase
)


class InPlayResult(dict):
    """Result Endpoint `results` array contents access.
    (e.g.)
        {
            "type": "CL",
            ...
            "CD": "1"
        }
    """

    def __init__(self, data: dict):
        """Constructor for InPlayResult."""
        super(InPlayResult, self).__init__(data)

    @property
    def type(self) -> str:
        """Access for `type`."""
        return self.get("type")


class InPlayEventsResponse(Bet365Response):
    """
    Response Object Facade for InPlay Events Endpoint.

    The object wraps the response and exposes dot notation access

    The top level accesses for `inplay` endpoint are:
        "success": int
        "results": list

    The `results` object is parsed into `InPlayResult` facades
    Say you have a parsed response from InPlayEvents Endpoint
    >>> response_object.results
    >>> [
    ...     {
    ...         "type":"CL",
    ...         "CD":"1",
    ...         "FF":"",
    ...         "ID":"1",
    ...         "IT":"OV_1_1_9",
    ...         "NA":"Soccer",
    ...         "OR":"0",
    ...         "PC":"3"
    ...        },
    ... ]

    >>> response_object.results[0].type
    >>> "67890"
    """

    def __init__(self, data):
        """Constructor for InPlayEventsResponse."""
        super(InPlayEventsResponse, self).__init__(data)
        self.stats = StatsBase(data)

    @property
    def results(self) -> Union[List[InPlayResult], None]:
        """Access for `results`."""
        if not self._results:
            return None

        parsed_results = []
        for record in self._results:
            parsed_results.append(InPlayResult(record))

        return parsed_results



"""
Bet365 API Wrapper.

Bet365 Serves as a Client to make requests to Bet365 API
see for documentation: (https://1394697259.gitbook.io/bet365-api/)

Bet365 Exposes 6 Endpoints:
    Result ["GET"]
    InPlay Filter ["GET"]
    InPlay Odds ["GET"]
    PreMatch odds ["GET"]
    InPlay Events ["GET"]
    Upcoming Events ["GET"]

Responses are parsed into Facade Access objects (Base Bet365Response)
"""
from typing import Optional, Union
from urllib.parse import urljoin

import requests

import pybet365.response as facades
from pybet365.response import Bet365Response
from pybet365.client.config import RESPONSE_OBJECT_FACTORY


class Bet365:
    """
    Bet365 API Wrapper.

    Accessible Endpoints:
        Bet365 Result ["GET"] via `result(...)`
        Bet365 InPlay Filter ["GET"] via `in_play_filter(...)`
        Bet365 InPlay Odds ["GET"] via `in_play_odds(...)`
        Bet365 PreMatch odds ["GET"] via `pre_match_odds(...)`
        Bet365 InPlay Events ["GET"] via `in_play_events(...)`
        Bet365 Upcoming Events ["GET"] via `upcoming_events(...)`
    """

    def __init__(self, api_host, api_key):
        """Constructor for Bet365."""
        self.base_url = "https://bet365-sports-odds.p.rapidapi.com/{}/bet365/"
        self.headers = {
            "x-rapidapi-host": api_host,
            "x-rapidapi-key": api_key,
        }

    def _get(
        self, url_extras: str, params: dict, version: str = "v1"
    ) -> Bet365Response:
        """
        Request maker for Bet365.

        NOTE: Internal method to be invoked by direct endpoint requests

        Args:
            url_extras (str): Tail for desired endpoint provided by caller
            params (dict): GET operation params dict
            version (str): API version for `url` provided by caller

        Returns:
            Bet365Response: Response object accessible by dot notation

        Raises:
            `raise_for_status()` for statuses other than `200`
        """
        url = urljoin(self.base_url.format(version), url_extras)

        response = requests.get(
            url=url, headers=self.headers, params=self._prune(params)
        )
        response.raise_for_status()

        try:
            # Using factory pattern we instantiate the Response Object
            delegation = getattr(
                facades, RESPONSE_OBJECT_FACTORY.get(url_extras)
            )

            # Load the desired `.json()` response to ResponseObject
            delegate_object = delegation(response.json())

        except AttributeError:
            # fall back on `.json()` if facade does not exist
            return response.json()

        return delegate_object

    def result(self, event_id: str) -> Bet365Response:
        """
        Caller for `Result` endpoint of Bet365 API.

        Args:
            event_id (str): Sporting event id to get result for

        Returns:
            Bet365Response: Response Object for `result` endpoint
        """
        querystring = {
            "event_id": event_id
        }

        return self._get(url_extras="result", params=querystring)

    def in_play_filter(
        self,
        sport_id: Optional[str] = None,
        league_id: Optional[str] = None
    ) -> Bet365Response:
        """
        Caller for `InPlay Filters` endpoint of Bet365 API.

        Args:
            sport_id (Optional[str]): Identifier for sport type
            league_id (Optional[str]): Identifier for specific league

        Returns:
            Bet365Response: Response Object for `in_play_filter` endpoint
        """
        querystring = {
            "sport_id": sport_id,
            "league_id": league_id
        }

        return self._get(url_extras="inplay_filter", params=querystring)

    def in_play_odds(
        self,
        fi: str,
        raw: Optional[str] = None,
        lineup: Optional[str] = None,
        stats: Optional[str] = None,
    ) -> Bet365Response:
        """
        Caller for `InPlay Odds` endpoint of Bet365 API.

        Args:
            fi (str): FI from Bet365 InPlay
            raw (Optional[str]): option for raw Bet365 body response
            lineup (Optional[str]): lineup info (NOTE: ONLY FOR CRICKET)
            stats (Optional[str]): extra stats info
                (NOTE: ONLY FOR SOCCER, BASKETBALL, CRICKET, BASEBALL, TENNIS)

        Returns:
            Bet365Response: Response Object for `in_play_odds` endpoint
        """
        querystring = {
            "FI": fi,
            "raw": raw,
            "lineup": lineup,
            "stats": stats
        }

        return self._get(url_extras="event", params=querystring)

    def pre_match_odds(
        self, fi: str, raw: Optional[str] = None
    ) -> Bet365Response:
        """
        Caller for `PreMatch Odds` endpoint of Bet365 API.

        Args:
            fi (str): FI from Bet365 InPlay
            raw (Optional[str]): option for raw Bet365 body response

        Returns:
            Bet365Response: Response Object for `pre_match_odds` endpoint
        """
        querystring = {
            "FI": fi,
            "raw": raw
        }

        return self._get(
            url_extras="prematch", params=querystring, version="v2"
        )

    def in_play_events(self, raw: Optional[str] = None) -> Bet365Response:
        """
        Caller for `InPlay Events` endpoint of Bet365 API.

        Args:
            raw (Optional[str]): option for raw Bet365 body response

        Returns:
            Bet365Response: Response Object for `in_play_events` endpoint
        """
        querystring = {
            "raw": raw
        }

        return self._get(url_extras="inplay", params=querystring)

    def upcoming_events(
        self,
        sport_id: str,
        page: Optional[str] = None,
        lng_id: Optional[str] = None,
        day: Optional[str] = None,
        league_id: Optional[str] = None,
    ) -> Bet365Response:
        """
        Caller for `Upcoming Events` endpoint of Bet365 API.

        Args:
            sport_id (str): String identifier for sport type
            page (Optional[str]): Pagination for API Response
                (NOTE: Out of box default is 50)
            lng_id (Optional[str]): Language Id
            day (Optional[str]): Go forward ONLY date to query
                (e.g.) - 20201124
            league_id (Optional[str]): Id for desired league

        Returns:
            Bet365Response: Response Object for `upcoming_events` endpoint
        """
        querystring = {
            "sport_id": sport_id,
            "page": page,
            "LNG_ID": lng_id,
            "day": day,
            "league_id": league_id,
        }

        return self._get(url_extras="upcoming", params=querystring)

    @staticmethod
    def _prune(params: dict) -> Union[dict, None]:
        """
        Cleaner of `params` dictionary for API Request

        If `value` for `key` is `NoneType` this key will be removed

        Args:
            params (dict): `params` dictionary for API request
            (e.g.)
                { "sport_id": "19", "lng_id": None }

        Returns:
            pruned_params (dict): `params` dict without `None` values or keys
            (e.g.)
                { "sport_id": "19" }
        """
        pruned_params = dict((k, v) for k, v in params.items() if v)

        return pruned_params



"""Mock Objects for pytest."""
from tests.utils import load_json

from requests import HTTPError


class MockRequestsResponse(object):
    """MockRequestsResponse object."""

    def __init__(self, filepath: str, status: int = 200):
        self.status = status
        self._path = filepath

    def raise_for_status(self):
        if self.status != 200:
            raise HTTPError

        return None

    def json(self):
        return load_json(self._path)



"""Unit tests for `betfund_bet365.client` modules."""
import mock
import pytest
import requests

from requests import HTTPError

from unittest import TestCase

from pybet365.client.client import Bet365

from tests.mocks import MockRequestsResponse


class TestBet365(TestCase):
    """Unit tests for Bet365 Client."""

    def setUp(self) -> None:
        """Instantiate Bet365."""
        self.test_client = Bet365(
            api_host="you-will-never-guess",
            api_key="you-will-never-guess"
        )

    def test_constructor(self):
        """Unit test for `Bet365().__init__(...)`."""
        assert self.test_client.headers == {
            "x-rapidapi-host": "you-will-never-guess",
            "x-rapidapi-key": "you-will-never-guess",
        }
        assert self.test_client.base_url == (
            "https://bet365-sports-odds.p.rapidapi.com/{}/bet365/"
        )

    @mock.patch.object(requests, "get")
    def test_get_raises(self, mock_api_response):
        """Unit test for `._get(...)` raises."""
        mock_api_response.return_value = MockRequestsResponse(
            filepath=None, status=404
        )

        with pytest.raises(HTTPError):
            self.test_client._get(
                url_extras="failure", params={"oops": "sorry"}
            )

    @mock.patch.object(requests, "get")
    def test_upcoming_events(self, mock_api_response):
        """Unit test for `._get(...)` success."""
        mock_api_response.return_value = MockRequestsResponse(
            filepath="testData/upcoming_events_table_tennis.json"
        )

        result = self.test_client._get(
            url_extras="upcoming", params={"params": "dict"}
        )

        assert isinstance(result, dict)



"""
Facade `BaseResponse Object` Delegation

Bet365 API Responses contain 2 common components
Bet365Response object is tasked to parse:
    "success": int
    "results": list

Further delegation for facade access to responses are subclassed

`InPlayFilterResponse`
`InPlayOddsResponse`
`PreMatchOddsResponse`
`PreMatchOddsResponse`
`InPlayEventsResponse`
`UpcomingEventsResponse`

The objects are accessible via dot notation or via `.get(..)`
"""
from typing import Union


class Bet365Response(dict):
    """Base ResponseObject creator for Bet365 API Response."""

    def __init__(self, data: dict):
        """Constructor for Bet365Response."""
        super(Bet365Response, self).__init__(data)

    @property
    def success(self) -> int:
        """Access for `success`."""
        return self.get("success")

    @property
    def _results(self) -> Union[list, None]:
        """Access for `results`."""
        # NOTE: This is overloaded hence the `_*`
        return self.get("results")


class FiResultBase(dict):
    """
    Component for dot notation for "FI" based objects.

    (e.g.)
    >>> data = {
    ...     "FI":"87967884",
    ...     "event_id":"12312412",
    ...     "main": {...}
    ... }

    >>> FiResultBase(data).fi
    >>> "87967884"
    """

    def __init__(self, data: dict):
        """Constructor for FiResultBase."""
        super(FiResultBase, self).__init__(data)

    @property
    def fi(self) -> str:
        """Access for `fi`."""
        return self.get("FI")

    @property
    def event_id(self) -> str:
        """Access for `event_id`."""
        return self.get("event_id")

    @property
    def main(self) -> dict:
        """Access for `main`."""
        return self.get("main")


class MetaBase(dict):
    """
    Component for dot notation for objects with "id" and "name".

    (e.g.)
    >>> data = {
    ...     "id":"10037409",
    ...     "name":"Mexico Liga MX Femenil"
    ... }

    >>> MetaBase(data).id
    >>> "10037409"
    """

    def __init__(self, data: dict):
        """Constructor for MetaBase."""
        super(MetaBase, self).__init__(data)

    @property
    def id(self) -> str:
        """Access for `id`."""
        return self.get("id")

    @property
    def name(self) -> str:
        """Access for `name`."""
        return self.get("name")

    @property
    def image_id(self) -> str:
        """Access for `image_id`."""
        return self.get("image_id")

    @property
    def cc(self) -> str:
        """Access for `cc`."""
        return self.get("cc")


class PagerBase(dict):
    """
    Component for dot notation access of `pager` object in API Response.
    (e.g.)
    >>> data = {
    ...     "page": 1,
    ...     "per_page": 50,
    ...     "total": 730
    ... }

    >>> PagerBase(data).page
    >>> 1

    >>> PagerBase(data).per_page
    >>> 50
    """

    def __init__(self, data: dict):
        """Constructor for PagerBase."""
        super(PagerBase, self).__init__(data)

    @property
    def page(self) -> Union[int, None]:
        """Access for `page`."""
        return self.get("page")

    @property
    def per_page(self) -> Union[int, None]:
        """Access for `per_page`."""
        return self.get("per_page")

    @property
    def total(self) -> Union[int, None]:
        """Access for `total`."""
        return self.get("total")


class ResultBase(dict):
    """
    Component for dot notation access of `results` from any Endpoint.

    (e.g.)
    >>> data = {
    ...   "id":"86576599",
    ...   "sport_id":"1",
    ...   "away":{
    ...       "id":"10361085",
    ...       "name":"Chivas Guadalajara Women"
    ...    },
    ...    "ss": None,
    ...    "our_event_id":"2130836",
    ...    "updated_at":"1581990232"
    ... }

    >>> ResultBase(data).id
    >>> "86576599"

    >>> ResultBase(data).away.id
    >>> "10361085"
    """

    def __init__(self, data: dict):
        """Constructor for ResultBase."""
        super(ResultBase, self).__init__(data)

    @property
    def id(self) -> str:
        """Access for `id`."""
        return self.get("id")

    @property
    def sport_id(self) -> str:
        """Access for `sport_id`."""
        return self.get("sport_id")

    @property
    def time(self) -> str:
        """Access for `time`."""
        return self.get("time")

    @property
    def time_status(self) -> str:
        """Access for `time_status`."""
        return self.get("time_status")

    @property
    def league(self) -> MetaBase:
        """Access for `league`."""
        return MetaBase(self.get("league"))

    @property
    def home(self) -> MetaBase:
        """Access for `home`."""
        return MetaBase(self.get("home"))

    @property
    def away(self) -> MetaBase:
        """Access for `away`."""
        return MetaBase(self.get("away"))

    @property
    def ss(self) -> str:
        """Access for `ss`."""
        return self.get("ss")


class StatsBase(dict):
    """
    Component for dot notation access of `stats` from any Endpoint.

    (e.g.)
    >>> data = {
    ...     "event_id":"2130389",
    ...     "update_at":"1581990853",
    ...     "update_dt":"2020-02-18 01:54:13"
    ... }

    >>> StatsBase(data).event_id
    >>> "2130389"

    >>> StatsBase(data).update_at
    >>> "1581990853"
    """

    def __init__(self, data: dict):
        """Constructor for StatsBase."""
        super(StatsBase, self).__init__(data)

    @property
    def event_id(self) -> str:
        """Access for `event_id`."""
        return self.get("event_id")

    @property
    def update_at(self) -> str:
        """Access for `update_at`."""
        return self.get("update_at")

    @property
    def update_dt(self) -> str:
        """Access for `update_dt`."""
        return self.get("update_dt")



"""Response namespace."""

from .base import (
    Bet365Response,
    FiResultBase,
    MetaBase,
    PagerBase
)

from .in_play_events import (
    InPlayEventsResponse,
    InPlayResult
)

from .pre_match_odds import PreMatchOddsResponse

from .result import (
    Result,
    ResultEvent,
    ResultResponse
)

from .upcoming_events import (
    UpcomingEvent,
    UpcomingEventsResponse
)

__all__ = [
    "Bet365Response",
    "FiResultBase",
    "InPlayEventsResponse",
    "InPlayResult",
    "MetaBase",
    "PagerBase",
    "PreMatchOddsResponse",
    "Result",
    "ResultEvent",
    "ResultResponse",
    "UpcomingEvent",
    "UpcomingEventsResponse"
]



"""
Facade for `upcoming_events` Delegation

Bet365 API Responses contain 2 common components
Bet365Response object is tasked to parse:
    "success": int
    "results": list

This module provides delegation for `ResultResponse`

The objects are accessible via dot notation or via `.get(..)`
"""

from typing import List, Union

from pybet365.response.base import (
    Bet365Response,
    MetaBase,
    ResultBase
)


class ResultEvent(dict):
    """Result Endpoint `events` array contents access.
    (e.g.)
        {
            "id": "1231251"
            "text": "1' - 1st Goal -   (Estudiantes Rio Cuarto) -"
        }
    """

    def __init__(self, data: dict):
        """Constructor for UpcomingEventsResponse."""
        super(ResultEvent, self).__init__(data)

    @property
    def id(self) -> str:
        """Access for `id`."""
        return self.get("id")

    @property
    def text(self):
        """Access for `text`."""
        return self.get("text")


class Result(ResultBase):
    """
    Subclass of ResultBase with additional fields for `Result` endpoint.

    (e.g.)
    >>> data = {
    ...   "id":"86576599",
    ...   "sport_id":"1",
    ...   "away":{
    ...       "id":"10361085",
    ...       "name":"Chivas Guadalajara Women"
    ...    },
    ...    "scores": {
    ...        "1": {
    ...            "home": "1",
    ...            "away": "2"
    ...        }
    ...    },
    ...    "updated_at": "1581990232"
    ... }

    >>> Result(data).id
    >>> "86576599"

    >>> Result(data).away.id
    >>> "10361085"
    """

    def __init__(self, data: dict):
        """Constructor for Result."""
        super(Result, self).__init__(data)

    @property
    def o_away(self) -> MetaBase:
        """Access for `o_away`."""
        return MetaBase(self.get("o_away"))

    @property
    def timer(self) -> dict:
        """Access for `timer`."""
        return self.get("timer")

    @property
    def scores(self) -> dict:
        """Access for `scores`."""
        return self.get("scores")

    @property
    def stats(self) -> dict:
        """Access for `stats`."""
        return self.get("stats")

    @property
    def extra(self) -> dict:
        """Access for `extra`."""
        return self.get("extra")

    @property
    def events(self) -> Union[List[ResultEvent], None]:
        """Access for `events`."""
        if not self.get("events"):
            return None

        parsed_results = []
        for record in self.get("events"):
            parsed_results.append(ResultEvent(record))

        return parsed_results

    @property
    def has_lineup(self) -> str:
        """Access for `has_lineup`."""
        return self.get("has_lineup")

    @property
    def inplay_created_at(self) -> str:
        """Access for `inplay_created_at`."""
        return self.get("inplay_created_at")

    @property
    def inplay_updated_at(self) -> str:
        """Access for `inplay_updated_at`."""
        return self.get("inplay_updated_at")

    @property
    def confirmed_at(self) -> str:
        """Access for `confirmed_at`."""
        return self.get("confirmed_at")


class ResultResponse(Bet365Response):
    """
    Response Object Facade for Result Endpoint.

    The object wraps the response and exposes dot notation access

    The top level accesses for `upcoming` endpoint are:
        "success": int
        "results": list

    The `results` object is parsed into `Result` facades
    Say you have a parsed response from UpcomingEvents Endpoint
    >>> response_object.results
    >>> [
    ...   {
    ...     "id": "67890",
    ...     "inplay_created_at": "1586461906"
    ...   },
    ... ]

    >>> response_object.results[0].id
    >>> "67890"

    >>> response_object.results[0].inplay_created_at
    >>> "1576465906"
    """

    def __init__(self, data):
        """Constructor for ResultResponse."""
        super(ResultResponse, self).__init__(data)

    @property
    def results(self) -> Union[List[Result], None]:
        """Access for `results`."""
        if not self._results:
            return None

        parsed_results = []
        for record in self._results:
            parsed_results.append(Result(record))

        return parsed_results
